@@ -6,25 +6,24 @@ various things.
 
 Copyright (c) 2003 Nelson Minar <nelson@monkey.org>
 http://www.nelson.monkey.org/~nelson/weblog/tech/phone/
-Copyright (c) 2005, 2006 Paul Boddie <paul@boddie.org.uk>
+Copyright (c) 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>
 http://www.boddie.org.uk/paul
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU Lesser General Public License as published by the Free
+Software Foundation; either version 3 of the License, or (at your option) any
+later version.
 
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+details.
 
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+You should have received a copy of the GNU Lesser General Public License along
+with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = "0.2.1"
+__version__ = "0.2.2"
 
 try:
     import serial # Requires pySerial http://pyserial.sourceforge.net/
@@ -753,27 +752,23 @@ class T616:
 
         self._expect("AT+CMGD=%s" % index, "\r\nOK\r\n")
 
-    def writeMessage(self, message, sender="", timeout=5):
+    def writeMessage(self, message, address="", timeout=5):
 
         """
-        Write the given 'message' (a Unicode object), optionally specifying the
-        'sender' (a Unicode object containing an international telephone number
-        without any non-numeric characters, such as the leading "+" character).
+        Write the given 'message' (a Unicode object), optionally specifying an
+        'address' (a Unicode object containing either a local telephone number
+        or an international telephone number with a leading "+" character).
+
         The optional 'timeout' (in seconds, default 5 seconds) specifies the
         length of time that should be used to wait for the storage of the
         message.
         """
 
-        # Switch character set and message format.
-
-        format = self.getMessageFormat()
-        charset = self.getCharacterSet()
-        self.setMessageFormat(self.TEXT)
-        self.setCharacterSet("UCS2")
+        format, charset = self._switchMessageFormat()
 
         try:
             self.ser.write('AT+CMGW')
-            self.ser.write('="%s"\r' % to_ucs2(sender))
+            self.ser.write('="%s"\r' % to_ucs2(address))
             response = self._readAll()
             if response != '\r\n> \x00':
                 raise CommandException("Could not understand response %s" % repr(response))
@@ -786,15 +781,68 @@ class T616:
             self.ser.setTimeout(timeout * 1000)
             response = self._readAll()
             self.ser.setTimeout(old_timeout)
+
+            # NOTE: The documentation claims that an index should be returned,
+            # NOTE: but the T610 just returns an "OK" message.
+
             if response != '\r\nOK\r\n':
                 raise CommandException("Could not understand response %s" % repr(response))
 
         finally:
+            self._resetMessageFormat(format, charset)
 
-            # Reset the character set and message format.
+    def sendMessage(self, message, recipient, timeout=5):
 
-            self.setCharacterSet(charset)
-            self.setMessageFormat(format)
+        """
+        Send the given 'message' (a Unicode object), specifying the 'recipient'
+        (a Unicode object containing either a local telephone number or an
+        international telephone number with a leading "+" character).
+
+        The optional 'timeout' (in seconds, default 5 seconds) specifies the
+        length of time that should be used to wait for the storage of the
+        message.
+        """
+
+        format, charset = self._switchMessageFormat()
+
+        try:
+            self.ser.write('AT+CMGS')
+            self.ser.write('="%s"\r' % to_ucs2(recipient))
+            response = self._readAll()
+            if response != '\r\n> \x00':
+                raise CommandException("Could not understand response %s" % repr(response))
+            self.ser.write(to_ucs2(message) + chr(26))
+            self._debug("(Message sent using AT+CMGS...)")
+
+            # Wait longer than usual for the result.
+
+            old_timeout = self.ser.getTimeout()
+            self.ser.setTimeout(timeout * 1000)
+            cmgsInfo = self._readAll()
+            self.ser.setTimeout(old_timeout)
+            m = re.search(r"\+CMGS: (\d+)(,\d\d/\d\d/\d\d,\d\d:\d\d:\d\d[-+]\d\d)?", cmgsInfo)
+            if not m:
+                raise CommandException("Couldn't parse CMGS info %s" % repr(cmgsInfo))
+
+        finally:
+            self._resetMessageFormat(format, charset)
+
+    def _switchMessageFormat(self):
+
+        "Switch character set and message format."
+
+        format = self.getMessageFormat()
+        charset = self.getCharacterSet()
+        self.setMessageFormat(self.TEXT)
+        self.setCharacterSet("UCS2")
+        return format, charset
+
+    def _resetMessageFormat(self, format, charset):
+
+        "Reset the character set and message format."
+
+        self.setCharacterSet(charset)
+        self.setMessageFormat(format)
 
     # Internal parsing methods.
 
